//
// Copyright (C) 2008-2009 Reto Gantenbein
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//

#include <MACAddress.h>
#include <Ieee802Ctrl_m.h>
#include "RAWEtherFrame.h"

Register_Class(RAWEtherFrame);

RAWEtherFrame::RAWEtherFrame(const char *name, int kind) : RAWEtherFrame_Base(name, kind)
{
	name = NULL;
	kind = 0;
}

RAWEtherFrame::RAWEtherFrame(const RAWEtherFrame& other) : RAWEtherFrame_Base(other.getName())
{
	operator=(other);
}

RAWEtherFrame& RAWEtherFrame::operator=(const RAWEtherFrame& other)
{
	RAWEtherFrame_Base::operator=(other);
	return *this;
}

/**
 * Sets the destination address of the packet from the destination MAC address of the packet
 */
void RAWEtherFrame::setDest()
{
    unsigned char destMac[ETH_ALEN];
    MACAddress macAddr;

    memmove(&destMac, rawBytes_var, ETH_ALEN);
    macAddr.setAddressBytes((unsigned char *) destMac);
//    printf("\nMACAddress: %02x:%02x:%02x:%02x:%02x:%02x\n", destMac[0], destMac[1], destMac[2], destMac[3], destMac[4], destMac[5]);

    Ieee802Ctrl *controlInfo = new Ieee802Ctrl();
    controlInfo->setDest(macAddr);
    setControlInfo(controlInfo);
}

/**
 * Set Ethernet frame payload (setRawBytesArraySize() must be called previously)
 * (avoid the expensive call of setRawBytes for every byte)
 */
void RAWEtherFrame::setPayload(const char *data)
{
	int payloadSize = getRawBytesArraySize();
	memmove(rawBytes_var, data, payloadSize);
}

/**
 * Get the Ethernet frame payload
 */
void RAWEtherFrame::getPayload(char *buf)
{
	int payloadSize = getRawBytesArraySize();
	memmove(buf, rawBytes_var, payloadSize);
}
